/**
 * @author Luc Yriarte
 * 
 * Licence : BSD http://www.opensource.org/licenses/bsd-license.php
 *  
 */
package basicSim;

/**
 * Vertical Cone, height h on the y axis, base of radius r approximated as a
 * polygon of n sides.
 */

public class PolyCone extends Object3D {

	double height, radius;

	int n_sides;

	/**
	 * @param nTotal segments for base circle approximation
	 * @param n segments to draw a partial base
	 * @param h height on the y axis
	 * @param r base radius
	 */
	public void init(int nTotal, int n, double h, double r) {
System.out.println("PolyCone "+ nTotal + " " + n + " " + h + " " + r + " ");
		height = h;
		radius = r;
		if (n < 3)
			n = 3;
		if (nTotal < n)
			nTotal = n;
		n_sides = n;
		nFacet = n_sides + 1;
		facet = new PolygonEdgeIndexes[nFacet];
		// polygon
		facet[n_sides] = new PolygonEdgeIndexes(this, n_sides);
		double dh = h / 2;
		double step = 2 * Math.PI / nTotal;
		Vector3D v[] = new Vector3D[n + 1];
		int[][] e = new int[n * 2][2];
		int i;
		for (i = 0; i < n; i++) {
			// polygon
			v[i] = new Vector3D(r * Math.cos(i * step), -dh, r
					* Math.sin(i * step));
			e[i][0] = i;
			e[i][1] = (i + 1) % n;
			// polygon facet
			facet[n_sides].edge[i] = i;
			// vertical lines
			e[i + n][0] = n;
			e[i + n][1] = i;
			// vertical triangle facets
			facet[i] = new PolygonEdgeIndexes(this, 3);
			facet[i].edge[0] = i;
			facet[i].edge[1] = (i + 1) % n;
			facet[i].edge[2] = i + n;
		}
		v[n] = new Vector3D(0, dh, 0);
		mesh = new Mesh(n + 1, n * 2, v, e);
	}

	public PolyCone(int n0, int n, double h, double r) {
		super();
		init(n0, n, h, r);
	}

	public PolyCone(int n, double h, double r) {
		super();
		init(n, n, h, r);
	}

	public PolyCone(double h, double r) {
		super();
		init(N_DEFAULT, N_DEFAULT, h, r);
	}

	public String strObject(int nTab) {
		String str = "", strTab = "";
		for (int i = 0; i < nTab; i++)
			strTab += "\t";
		str += strTab + "{ cone r=" + new Double(radius).toString() + " h="
				+ new Double(height).toString();
		if (n_sides != N_DEFAULT)
			str += " n=" + new Integer(n_sides).toString();
		str += "\n";
		str += strCoord(nTab);
		str += strColor(nTab);
		str += strChild(nTab);
		str += strTab + "}\n";
		return str;
	}

}
