/**
 * @author Luc Yriarte
 * 
 * Licence : BSD http://www.opensource.org/licenses/bsd-license.php
 *  
 */
package basicSim;

/**
 * Column vector
 */
public class Vector extends Matrix {

	static public Vector MatrixAsVector(Matrix aMatrix) {
		return new Vector(aMatrix.nLines, aMatrix.cell);
	}
	
	public Vector() {
		super();
	}

	public Vector(int lines) {
		super(lines, 1);
	}

	public Vector(int lines, double[][] cell) {
		super(lines, 1, cell);
	}

	public Vector(Matrix aMatrix) {
		super(aMatrix.nLines, 1);
		int i;
		for (i = 0; i < nLines; i++) {
			this.cell[i][0] = aMatrix.cell[i][0];
		}
	}
	
	public Vector(Matrix aMatrix, int column) {
		super(aMatrix.nLines, 1);
		int i;
		for (i = 0; i < nLines; i++) {
			this.cell[i][0] = aMatrix.cell[i][column];
		}
	}
	
	/**
	 * noted "|a|"
	 * @return square root of the sum of squares
	 */
	public double norm() {
		double n = 0;
		int i;
		for (i = 0; i < this.nLines; i++) {
			n += this.cell[i][0] * this.cell[i][0];
		}
		n = Math.sqrt(n);
		return n;
	}

	/**
	 * noted "a . b"
	 * @param aVector
	 * @return this vector transposed multiplied by a vector
	 */
	public double scalProd(Vector aVector) {
/*		Matrix mResult = this.transpose();
		mResult = mResult.mul(aVector);
		return mResult.getCell()[0][0];
*/
		double result = 0;
		int i;
		for (i = 0; i < nLines; i++) {
			result += this.cell[i][0] * aVector.cell[i][0];
		}
		return result;		
	}

	/**
	 * noted ""
	 * @return ||=1 unit vector in this vector direction 
	 */
	public Vector unitVector() {
		double n = this.norm();
		int i;
		Vector vResult = new Vector(nLines);
		if (n != 0) {
			for (i = 0; i < nLines; i++) {
				vResult.cell[i][0] = this.cell[i][0] / n;
			}
		}
		return vResult;
	}

	public double vectAngle(Vector aVector) {
		double prodNorm, prodScalar;
		prodScalar = this.scalProd(aVector);
		prodNorm = this.norm() * aVector.norm();
		return Math.acos(prodScalar/prodNorm);
	}

	/**
	 * noted "a x b"
	 * @param aVector
	 * @return vector product
	 */
	public Vector vectProd(Vector aVector) {
		Vector vResult = new Vector(aVector.nLines);
		int i,ia,ib;
		for (i = 0; i < nLines; i++) {
			ia = (i + 1) % nLines;
			ib = (ia+ 1) % nLines;
			vResult.cell[i][0] = this.cell[ia][0] * aVector.cell[ib][0] - this.cell[ib][0] * aVector.cell[ia][0];
		}
		return vResult;
	}

	/**
	 * noted "a x b"
	 * @param aVector
	 * @return vector division
	 */
	public Vector vectDiv(Vector aVector) {
		Vector vResult = new Vector();
		if (this.isZero()) {
			if (aVector.isZero())
				return vResult;
			else
				return null;
		}
		double norma = this.norm();
		vResult = aVector.vectProd(this);
		vResult = new Vector(vResult.mulNum(1/(norma*norma)));
		return vResult;
	}

}

