/**
 * @author Luc Yriarte
 * 
 * Licence : BSD http://www.opensource.org/licenses/bsd-license.php
 *  
 */
package physicSim;

import basicSim.Vector;
import basicSim.VectorXYZ;

/**
 * PointMass : A weighted 3D Object 
 * 
 * Physics units in m / kg / s
 * Left handed geometric coordinates
 * 
 */
public class PointMass {
	/** By default objects weight 1 kg */
	public static double DEFAULT_MASS=1;
	/** mass in kg */
	protected double mass;
	protected double invmass;
	/** */
	protected Vector position;
	protected Vector velocity;
	protected Vector acceleration;
	
	/**
	 * 
	 */
	public PointMass() {
		super();
		this.mass = DEFAULT_MASS;
		this.position=new VectorXYZ();
		this.velocity=new VectorXYZ();
		this.acceleration=new VectorXYZ();
		this.invmass = 1/this.mass;
	}
	
	/**
	 * @param mass
	 */
	public PointMass(double mass) {
		super();
		this.mass = mass;
		this.position=new VectorXYZ();
		this.velocity=new VectorXYZ();
		this.acceleration=new VectorXYZ();
		this.invmass = 1/this.mass;
	}
	
	/**
	 * Creates a point - mass with 0 speed and acceleration.
	 * @param mass
	 * @param position
	 */
	public PointMass(double mass, Vector position) {
		super();
		this.mass = mass;
		this.position = position;	
		this.velocity=new VectorXYZ();
		this.acceleration=new VectorXYZ();
		this.invmass = 1/this.mass;
System.out.println("PointMass : mass=" + mass + " x=" + position.getCell()[0][0] 
		+ " y=" + position.getCell()[1][0] 
		+ " z=" + position.getCell()[2][0]);		
	}
	
	/**
	 * @return Returns the mass.
	 */
	public double getMass() {
		return mass;
	}
	/**
	 * @param mass The mass to set.
	 */
	public void setMass(double mass) {
		this.mass = mass;
		this.invmass = 1/this.mass;
	}
	
	public Vector getPosition() {
		return position;
	}
	
	public void setPosition(Vector position) {
		this.position = position;		
	}
	
	public Vector getVelocity() {
		return velocity;
	}
	
	public Vector getAcceleration() {
		return acceleration;
	}

	/**
	 * @param dt
	 */
	public PointMass updateState(double dt) {
		velocity.addMulNumThis(acceleration, dt);
		position.addMulNumThis(velocity, dt);
		acceleration.toZero();
		return this;
	}
	
	/**
	 * @param f
	 */
	public PointMass applyForce(Vector f) {
		acceleration.addMulNumThis(f, invmass);
		return this;
	}
	
}
