/**
 * @author Luc Yriarte
 * 
 * Licence : BSD http://www.opensource.org/licenses/bsd-license.php
 *  
 */
package physicSim;

import basicSim.Object3D;
import basicSim.Vector;
import basicSim.Vector3D;

/**
 * SolidObject : A weighted 3D Object 
 * 
 * Physics units in m / kg / s
 * Left handed geometric coordinates
 * 
 */
public class SolidObject extends PointMass {
	/** Geometric object to draw.  */
	public Object3D object3D;
	
	/**
	 * 
	 */
	public SolidObject() {
		super();
	}
	
	/**
	 * @param mass
	 */
	public SolidObject(double m) {
		super(m);
	}
	
	/**
	 * @param mass
	 * @param object3D
	 */
	public SolidObject(double m, Object3D object3D) {
		super(m);
		this.object3D = object3D;
	}

	public void setPosition(Vector position) {
		super.setPosition(position);
		object3D.setPosition(new Vector3D(position).positionMatrix());
		
	}

	/**
	 * @param dt
	 */
	public PointMass updateState(double dt) {
		super.updateState(dt);
		object3D.setPosition(new Vector3D(position).positionMatrix());
		return this;
	}
		
}
