/**
 * @author Luc Yriarte
 * 
 * Licence : BSD http://www.opensource.org/licenses/bsd-license.php
 *  
 */
package physicSim;

import basicSim.Vector;
import basicSim.VectorXYZ;

public class Spring extends ForceGenerator {
	/** spring length at rest */
	public double length=1;
	public double stiffness=0.01;
	public Vector stretch;
	
	/**
	 * 
	 */
	public Spring() {
		super();
	}

	/**
	 * @param stretch
	 * default stiffness is 1. 
	 * length is the norm of the stretch vector
	 */
	public Spring(Vector stretch) {
		super();
		this.stiffness = 1;
		this.stretch = stretch;
		this.length = stretch.norm();
System.out.println("Spring length " + length);
	}
	
	/**
	 * @param stiffness
	 * @param stretch
	 * length is the norm of the stretch vector
	 */
	public Spring(double stiffness, Vector stretch) {
		super();
		this.stiffness = stiffness;
		this.stretch = stretch;
		this.length = stretch.norm();
	}
	
	/**
	 * @param length
	 * @param stiffness
	 * vertical stretch vector of length descending the y axis
	 */
	public Spring(double length, double stiffness) {
		super();
		this.length = length;
		this.stiffness = stiffness;
		this.stretch = new VectorXYZ(0,-length,0);
	}
	
	public Vector getForceVector() {
		return Vector.MatrixAsVector(stretch.unitVector().mulNumThis(-stiffness*(stretch.norm()-length)));
	}
}
