/**
 * @author Luc Yriarte
 * 
 * Licence : BSD http://www.opensource.org/licenses/bsd-license.php
 *  
 */
package physicSim;

import basicSim.Mesh;
import basicSim.Vector;
import basicSim.Vector3D;
import basicSim.VectorXYZ;

/**
 * A spring on every edge of the mesh.
 */
public class SpringMesh extends Mesh {

	Spring springEdge[];
	PointMass springPoint[];
	
	/**
	 * 
	 */
	public SpringMesh() {
		super();
		springEdge = null;
		springPoint = null;
	}

	/**
	 * @param aMesh references the mesh vertices and edges
	 */
	public SpringMesh(Mesh aMesh) {
		super(aMesh.nVertex, aMesh.nEdge, aMesh.vertex, aMesh.edge);
		init();
	}

	/**
	 * @param nV
	 * @param nE
	 * @param v
	 * @param e
	 */
	public SpringMesh(int nV, int nE, Vector3D[] v, int[][] e) {
		super(nV, nE, v, e);
		init();
	}

	public SpringMesh init() {
		int i;
//System.out.println("SpringMesh init obj " + nVertex + " edge " + nEdge);
		springEdge = new Spring[nEdge];
		VectorXYZ v0,v1;
		for (i=0; i<nEdge; i++) {
			v0=VectorXYZ.MatrixAsVectorXYZ(vertex[edge[i][0]]);
			v1=VectorXYZ.MatrixAsVectorXYZ(vertex[edge[i][1]]);
			// Initial spring position is not stretched
			springEdge[i]=new Spring(new VectorXYZ(v0.getX() - v1.getX(),
					v0.getY() - v1.getY(),
					v0.getZ() - v1.getZ()));
		}
		springPoint = new PointMass[nVertex];
		// PointMass <x,y,z> and Vector3D <x,y,z,1> reference the same cells
		for (i=0; i<nVertex; i++) {
			springPoint[i] = new PointMass(1, VectorXYZ.MatrixAsVectorXYZ(vertex[i]));
		}
		return this;
	}
	
	public SpringMesh update(double dt) {
		int i;
//System.out.println("SpringMesh update dt " + dt + ", obj " + nVertex + " edge " + nEdge);
		// Calculate spring length
		Vector v0,v1;
		for (i=0; i<nEdge; i++) {
			v0=vertex[edge[i][0]];
			v1=vertex[edge[i][1]];
			springEdge[i].stretch = VectorXYZ.MatrixAsVectorXYZ(v0.sub(v1));
		}
		// Apply each spring force on edge's both ends 
		for (i=0; i<nEdge; i++) {
			springPoint[edge[i][0]].applyForce(Vector.MatrixAsVector(springEdge[i].getForceVector().mulNum(0.5 * dt)));
			springPoint[edge[i][1]].applyForce(Vector.MatrixAsVector(springEdge[i].getForceVector().mulNum(-0.5* dt)));
		}
		// Update <x,y,z> of edge's both ends Vector3D in place 
		for (i=0; i<nVertex; i++) {
			springPoint[i].updateState(dt);
		}
		return this;
	}

}
