/**
 * @author Luc Yriarte
 * 
 * Licence : BSD http://www.opensource.org/licenses/bsd-license.php
 *  
 */
package springMesh;

import java.applet.Applet;
import java.awt.Color;
import java.awt.Event;
import java.awt.Graphics;

import physicSim.SolidObject;
import physicSim.SpringMesh;
import physicSim.SpringWorld;
import basicSim.BackBufferedGraphics;
import basicSim.Box;
import basicSim.Grid;
import basicSim.Object3D;
import basicSim.PolySphere;
import basicSim.Vector3D;

public class SpringMeshApplet extends Applet {

	static int NAV_ROTATION = 0, NAV_TRANSLATION = 1, NAV_ZOOM = 2;
	static int DEFAULT_FOCAL = 200, DEFAULT_X = 0, DEFAULT_Y = -10, DEFAULT_Z = 30, DEFAULT_MS = 83;

	int width, height;
	int dragX, dragY;
	
	SpringWorld world;
	Object3D model;
	SpringMesh sm;
	double ballsecs,deltasec;


	public SpringMeshApplet() {
		super();
	}

	public void init() {
		Object3D obj;
		width = this.getSize().width;
		height = this.getSize().height;
		
		world = new SpringWorld();
		world.space.trnZ = DEFAULT_Z;
		world.space.trnY = DEFAULT_Y;
		world.space.trnX = DEFAULT_X;
		world.simView = new BackBufferedGraphics(world.space, this.createImage(width,height), width, height, DEFAULT_FOCAL);
		world.view = this;
		world.millisec = DEFAULT_MS;
/*		obj = new Grid(2,30);
		obj.trnX=-30;
		obj.trnZ=-30;
		world.space.addChild(obj);
		obj = new Box(1,1,1);
		world.space.addChild(obj);
*/
		ballsecs=0;
		deltasec=0.66666;
		
		// ---- ---- 
//		world.simView.drawFacets=false;
		
		// ---- ---- 
		world.setG(0);
		SolidObject newSolid;
		model =  new PolySphere(20);
		model.color = new Color((int)(Math.random()*128),(int)(Math.random()*128),(int)(Math.random()*128));
		newSolid = new SolidObject(1, model);
		sm = new SpringMesh(model.getMesh());
		world.addObject(newSolid);
		world.springs = sm;
		
		// ---- ---- update and paint
		world.start();
	}
	
	public void paint(Graphics g) {
		world.simView.updateGraphics(g);
	}
	
	public void update(Graphics g) {
		this.paint(g);
	}

	public boolean mouseDown(Event arg0, int arg1, int arg2) {
		dragX = arg1;
		dragY = arg2;
		model.selectVertex(arg1,arg2,10);
		return true;
	}

	public boolean mouseDrag(Event arg0, int arg1, int arg2) {
		double deltaX, deltaY, deltaZ;
		deltaX = (arg1 - dragX);
		deltaY = (dragY - arg2);
		Vector3D v = model.getSelectedVertex();
		if (v != null) {
			v.setX(v.getX()+deltaX);
			v.setZ(v.getZ()+deltaY);
		}
		repaint();
		dragX = arg1;
		dragY = arg2;
		return true;
	}

	public boolean mouseUp(Event arg0, int arg1, int arg2) {
		dragX = dragY = 0;
		return true;
	}

	
}
